/**
 * 4 RGB LED Controller Video Display
 * Steven Cholewiak
 * http://www.semifluid.com
 *
 * Based upon: Pixelate by Hernando Barragan. 
 */


import processing.video.*;
import processing.serial.*;
import processing.core.*;
PFont font;

int[][] controllerAddress = {  {1, 2, 3, 4},
                               {5, 6, 7, 8},
                               {9, 10, 11, 12},
                               {13, 14, 15, 16}  };
int[][] ledRed = new int[2][2];
int[][] ledGreen = new int[2][2];
int[][] ledBlue = new int[2][2];

int hSize = 4*2;
int vSize = 4*2;
byte[] outputData = new byte[hSize/2*vSize/2*9];
int blockSize = 20;
Movie myMovie;
color[][] myMovieColors;
String input = "";
Serial myPort;

void setup() {  
  // Setup display for computer
  size(hSize * blockSize, vSize * blockSize, P2D);
  noStroke();
  background(0);
  
  String portName = Serial.list()[0];
  myPort = new Serial(this, portName, 19200);
  
  frameRate(60);
  smooth();
  font = createFont("Arial Bold",24);
  
  myMovie = new Movie(this, "RGBTestSmall.mov");
  myMovie.loop();
  myMovieColors = new color[hSize][vSize];
}

// Display values from movie
void draw()  {
  if(myMovie.available()) {
    myMovie.read();
    myMovie.loadPixels();
    
    for (int j = 0; j < vSize; j++) {
      for (int i = 0; i < hSize; i++) {
        myMovieColors[i][j] = myMovie.get(i * myMovie.width/hSize, j * myMovie.height/vSize);
      }
    }
  }
  
  for (int j = 0; j < vSize/2; j++) {
    for (int i = 0; i < hSize/2; i++) {
      for (int y = 0; y < 2; y++) {
        for (int x = 0; x < 2; x++) {
          ledRed[x][y] =   (myMovieColors[i*2+x][j*2+y] >> 16) & 0xFF;
          ledGreen[x][y] = (myMovieColors[i*2+x][j*2+y] >> 8) & 0xFF;
          ledBlue[x][y] =   myMovieColors[i*2+x][j*2+y] & 0xFF;
          fill(myMovieColors[i*2+x][j*2+y]);
          rect((i*2+x)*blockSize, (j*2+y)*blockSize, blockSize-1, blockSize-1);
        }
      }
      outputData[(j*vSize/2 + i)*9 + 0] = byte(0xAA);
      outputData[(j*vSize/2 + i)*9 + 1] = byte(0x10);
      outputData[(j*vSize/2 + i)*9 + 2] = byte((ledRed[0][0] & 0xF0) + (ledGreen[0][0] >> 4));
      outputData[(j*vSize/2 + i)*9 + 3] = byte((ledBlue[0][0] & 0xF0) + (ledRed[1][0] >> 4));
      outputData[(j*vSize/2 + i)*9 + 4] = byte((ledGreen[1][0] & 0xF0) + (ledBlue[1][0] >> 4));
      outputData[(j*vSize/2 + i)*9 + 5] = byte((ledRed[0][1] & 0xF0) + (ledGreen[0][1] >> 4));
      outputData[(j*vSize/2 + i)*9 + 6] = byte((ledBlue[0][1] & 0xF0) + (ledRed[1][1] >> 4));
      outputData[(j*vSize/2 + i)*9 + 7] = byte((ledGreen[1][1] & 0xF0) + (ledBlue[1][1] >> 4));
      outputData[(j*vSize/2 + i)*9 + 8] = byte(controllerAddress[i][j]);
    }
  }
  myPort.write(outputData);
  
  textFont(font,12);
  fill(255);
  text(frameRate,20,20); // Draw framerate
}

void mousePressed() {
  myPort.write(0xAA);
  myPort.write(0x21);
  myPort.write(0x11);
  myPort.write(0xFF);
  exit(); 
}
